--[[----------------------------------------------------------------------
      DataGatherer Module - Part of VanasKoS
Handles all external Player Data from Chat and Target Changes/Mouseovers
------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoSDataGatherer");

VanasKoSDataGatherer = VanasKoS:NewModule("DataGatherer");

local VanasKoSDataGatherer = VanasKoSDataGatherer;

L:RegisterTranslations("enUS", function() return {
	["Data Gathering"] = true,
	["Combat Log Range"] = true,
	["Adjust Combat Log Ranges"] = true,
	
	["Death Log Range"] = true,
	["Changes the logging range of the Death Log (in yards)"] = true,
	["Creature Log Range"] = true,
	["Changes the logging range of the Creature Log (in yards)"] = true,
	["Friendly Players Log Range"] = true,
	["Changes the logging range of the Friendly Players Log (in yards)"] = true,
	["Friendly Players Pets Log Range"] = true,
	["Changes the logging range of the Friendly Players Pets Log (in yards)"] = true,
	["Hostile Players Log Range"] = true,
	["Changes the logging range of the Hostile Players Log (in yards)"] = true,
	["Hostile Players Pets Log Range"] = true,
	["Changes the logging range of the Hostile Players Pets Log (in yards)"] = true,
	["Party Log Range"] = true,
	["Changes the logging range of the Party Log (in yards)"] = true,
	["Party Pets Log Range"] = true,
	["Changes the logging range of the Party Pets Log (in yards)"] = true,
} end);

L:RegisterTranslations("koKR", function() return {
	["Data Gathering"] = "데이터 수집",
	["Combat Log Range"] = "전투 로그 범위",
	["Adjust Combat Log Ranges"] = "전투 로그 범위를 조절합니다.",
	
	["Death Log Range"] = "죽음 로그 범위",
	["Changes the logging range of the Death Log (in yards)"] = "죽음 로그의 로깅 범위를 변경합니다.",
	["Creature Log Range"] = "NPC 로그 범위",
	["Changes the logging range of the Creature Log (in yards)"] = "NPC 로그의 로깅 범위를 변경합니다.",
	["Friendly Players Log Range"] = "우호적 플레이어 로그 범위",
	["Changes the logging range of the Friendly Players Log (in yards)"] = "우호적 플레이어 로그의 로깅 범위를 변경합니다.",
	["Friendly Players Pets Log Range"] = "우호적 플레이어 소환수 로그 범위",
	["Changes the logging range of the Friendly Players Pets Log (in yards)"] = "우호적 플레이어 소환수 로그의 로깅 범위를 변경합니다.",
	["Hostile Players Log Range"] = "적대적 플레이어 로그 범위",
	["Changes the logging range of the Hostile Players Log (in yards)"] = "적대적 플레이어 로그의 로깅 범위를 변경합니다.",
	["Hostile Players Pets Log Range"] = "적대적 플레이어 소환수 로그 범위",
	["Changes the logging range of the Hostile Players Pets Log (in yards)"] = "적대적 플레이어 소환수 로그의 로깅 범위를 변경합니다.",
	["Party Log Range"] = "파티 로그 범위",
	["Changes the logging range of the Party Log (in yards)"] = "파티 로그의 로깅 범위를 변경합니다.",
	["Party Pets Log Range"] = "파티 소환수 로그 범위",
	["Changes the logging range of the Party Pets Log (in yards)"] = "파티 소환수 로그의 로깅 범위를 변경합니다.",
} end);

local zone = "";

function VanasKoSDataGatherer:OnInitialize()
	VanasKoS:RegisterDefaults("DataGatherer", "realm", {
		data = {
			players = {
			},
			guilds = {
			},
		},
	});

	self.db = VanasKoS:AcquireDBNamespace("DataGatherer");

	-- import of old data, will be removed in some version in the future
	if(VanasKoS.db.realm.data) then
		self.db.realm.data = VanasKoS.db.realm.data;
		VanasKoS.db.realm.data = nil;
	end

	VanasKoS:RegisterList(nil, "PLAYERDATA", nil, self);
	VanasKoS:RegisterList(nil, "GUILDDATA", nil, self);

	VanasKoSGUI:AddConfigOption("DataGatherer", {
		type = 'group',
		name = L["Data Gathering"],
		desc = L["Data Gathering"],
		args = {
			clogrange = {
				type = 'group',
				name = L["Combat Log Range"],
				desc = L["Adjust Combat Log Ranges"],
				args = {
					deathlog = {
						type = 'range',
						name = L["Death Log Range"],
						desc = L["Changes the logging range of the Death Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatDeathLogRange", val); end,
						get = function() return GetCVar("CombatDeathLogRange"); end,
					},
					creaturelog = {
						type = 'range',
						name = L["Creature Log Range"],
						desc = L["Changes the logging range of the Creature Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangeCreature", val); end,
						get = function() return GetCVar("CombatLogRangeCreature"); end,
					},
					friendlylog = {
						type = 'range',
						name = L["Friendly Players Log Range"],
						desc = L["Changes the logging range of the Friendly Players Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangeFriendlyPlayers", val); end,
						get = function() return GetCVar("CombatLogRangeFriendlyPlayers"); end,
					},
					friendlypetlog = {
						type = 'range',
						name = L["Friendly Players Pets Log Range"],
						desc = L["Changes the logging range of the Friendly Players Pets Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangeFriendlyPlayersPets", val); end,
						get = function() return GetCVar("CombatLogRangeFriendlyPlayersPets"); end,
					},
					hostilelog = {
						type = 'range',
						name = L["Hostile Players Log Range"],
						desc = L["Changes the logging range of the Hostile Players Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangeHostilePlayers", val); end,
						get = function() return GetCVar("CombatLogRangeHostilePlayers"); end,
					},
					hostilepetlog = {
						type = 'range',
						name = L["Hostile Players Pets Log Range"],
						desc = L["Changes the logging range of the Hostile Players Pets Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangeHostilePlayersPets", val); end,
						get = function() return GetCVar("CombatLogRangeHostilePlayersPets"); end,
					},
					partylog = {
						type = 'range',
						name = L["Party Log Range"],
						desc = L["Changes the logging range of the Party Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangeParty", val); end,
						get = function() return GetCVar("CombatLogRangeParty"); end,
					},
					partypetlog = {
						type = 'range',
						name = L["Party Pets Log Range"],
						desc = L["Changes the logging range of the Party Pets Log (in yards)"],
						min = 20,
						max = 150,
						step = 10,
						set = function(val) SetCVar("CombatLogRangePartyPet", val); end,
						get = function() return GetCVar("CombatLogRangePartyPet"); end,
					},
				},
			},
		},
	});
end

function VanasKoSDataGatherer:OnEnable()
	-- Mouseover, Targetchanges
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");

	-- Chat Messages
	self:RegisterEvent("CHAT_MSG_HOSTILEPLAYER_BUFF", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_HOSTILEPLAYER_DAMAGE", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "ChatCombatMessageHandlerEnemy");

	-- Chat Messages Friendly - to gather messages & friendly targets
	self:RegisterEvent("CHAT_MSG_FRIENDLYPLAYER_BUFF", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_FRIENDLYPLAYER_DAMAGE", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", "ChatCombatMessageHandlerFriendly");

	-- say, emotes
	self:RegisterEvent("CHAT_MSG_SAY", "ChatMessageAndEmoteHandler");
	self:RegisterEvent("CHAT_MSG_EMOTE", "ChatMessageAndEmoteHandler");
	self:RegisterEvent("CHAT_MSG_TEXT_EMOTE", "ChatMessageAndEmoteHandler");
	
	-- on zonechange update zone
	self:RegisterEvent("ZONE_CHANGED", "UpdateZone");
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "UpdateZone");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "UpdateZone");
	
	self:RegisterEvent("VanasKoS_Player_Data_Gathered", "Data_Gathered");
	self:RegisterEvent("VanasKoS_Player_Detected", "Player_Detected");

	self:UpdateZone();
end

function VanasKoSDataGatherer:OnDisable()
	self:UnregisterAllEvents();
end

-- if a player was detected nearby through chat or mouseover, update lastseen
function VanasKoSDataGatherer:Player_Detected(name, guild, faction)
	-- if the player is on a relevant list, update last seen
	self:UpdateLastSeen(name);
end

function VanasKoSDataGatherer:IsOnList(listname, name, guild)
	local listVar = VanasKoS:GetList(listname);
	if(listVar and listVar[name]) then
		local creator = nil;
		if(listVar[name].creator) then
			creator = listVar[name].creator;
		end
		return listVar[name].reason, creator;
	else
		return nil;
	end
end


function VanasKoSDataGatherer:GetList(list)
	if(list == "PLAYERDATA") then
		return self.db.realm.data.players;
	elseif(list == "GUILDDATA") then
		return self.db.realm.data.guilds;
	else
		return nil;
	end
end

function VanasKoSDataGatherer:AddEntry(list, name, data)
	return true;
end

function VanasKoSDataGatherer:RemoveEntry(listname, name)
	local list = VanasKoS:GetList(listname);
	if(list and list[name]) then
		list[name] = nil;
		self:TriggerEvent("VanasKoS_List_Entry_Removed", listname, name);
	end
end

function VanasKoSDataGatherer:UpdateLastSeen(name)
	local list = VanasKoS:GetList("PLAYERDATA");
	if(list[string.lower(name)] ~= nil) then
		list[string.lower(name)].lastseen = time();
	end
end

function VanasKoSDataGatherer:Data_Gathered(list, name, data)
	-- self:PrintLiteral(list, name, guild, level, race, class, gender);
	if(name == nil or list == nil) then
		return;
	end
	
	local lname = name:lower();
	
	if(list ~= nil and name ~= nil) then
		local listVar = VanasKoS:GetList("PLAYERDATA");
		
		if(not listVar[lname]) then
			listVar[lname] = { };
		end
		self:UpdateLastSeen(name);
		
		listVar[lname].displayname = name;
		listVar[lname].guild = data['guild'];
		if(data['level'] and data['level'] ~= -1 or
			(data['level'] == -1 and listVar[lname].level ~= nil)) then
			listVar[lname].level = data['level'];
		end
		listVar[lname].race = data['race'];
		listVar[lname].class = data['class'];
		listVar[lname].gender = data['gender'];
		listVar[lname].zone = data['zone'];
		
		if(guild and VanasKoS:IsOnList("GUILDKOS", guild)) then
			local guildData = VanasKoS:GetList("GUILDDATA");
			if(not guildData[guild:lower()]) then
				guildData[guild:lower()] = { };
			end
			guildData[guild:lower()].displayname = guild;
		end
		
		VanasKoSGUI:Update();
	end
end

local continent = -1;
local zoneID = -1;
local zoneNames = nil;
local inBattleground = false;
local tourist = AceLibrary("Tourist-2.0")

function VanasKoSDataGatherer:UpdateZone()
	continent = GetCurrentMapContinent();
	zoneID = GetCurrentMapZone();
	zoneNames = { GetMapZones(continent) };
	zone = zoneNames[zoneID];
	
	if(continent == -1) then
		zone = GetZoneText();
	end

	if(zone == nil) then
		return;
	end

	if(tourist:IsBattleground(zone) or tourist:IsInstance(zone) or tourist:IsArena(zone)) then
		inBattleground = true;
	else
		inBattleground = false;
	end

	self:TriggerEvent("VanasKoS_Zone_Changed", zone, continent, zoneID);
end

function VanasKoSDataGatherer:IsInShattrath()
	if(continent == 3 and zoneID == 6) then
		return true;
	else
		return false;
	end
end

function VanasKoSDataGatherer:IsInBattleground()
	return inBattleground;
end

function VanasKoSDataGatherer:PLAYER_TARGET_CHANGED()
	if(UnitIsPlayer("target")) then
		local name = nil;
		local guild = nil;
		local faction = nil;
		
		name = UnitName("target");
		guild = GetGuildInfo("target");

		if(name == nil) then
			return;
		end
		
		local data = self:Get_Player_Data("target");

		local faction = nil;
		if(VanasKoS:IsKoSPlayer(name) or VanasKoS:IsKoSGuild(guild)) then
			faction = "kos";
		elseif(UnitExists("target") and UnitIsEnemy("player", "target")) then
			faction = "enemy";
		else
			faction = "friendly";
		end
		
		if(name ~= nil) then
			self:TriggerEvent("VanasKoS_Player_Detected", name, guild, faction, data);
		end
		
		self:TriggerEvent("VanasKoS_Player_Target_Changed", name, guild, faction);
	else
		self:TriggerEvent("VanasKoS_Mob_Target_Changed", nil, nil, nil);
	end
end

function VanasKoSDataGatherer:Get_Player_Data(unit)
	local data = nil;
	if(UnitIsPlayer(unit)) then
		local name = UnitName(unit);
		local guild = GetGuildInfo(unit);
		local level = UnitLevel(unit);
		local race = UnitRace(unit);
		local class = UnitClass(unit);
		local gender = UnitSex(unit);
		
		if(name == nil) then
			return;
		end
		local reason, list = VanasKoS:IsOnList(nil, name, guild);
		data = { ['guild'] = guild, 
					['level'] = level, 
					['race'] = race, 
					['class'] = class, 
					['gender'] = gender, 
					['zone'] = zone };
		self:TriggerEvent("VanasKoS_Player_Data_Gathered", list, name, data);
	end
	return data;
end

function VanasKoSDataGatherer:UPDATE_MOUSEOVER_UNIT()
	if(UnitIsPlayer("mouseover")) then
		local name = UnitName("mouseover");
		local guild = GetGuildInfo("mouseover");

		local data = self:Get_Player_Data("mouseover");
		
		local faction = nil;
		if(VanasKoS:IsKoSPlayer(name) or VanasKoS:IsKoSGuild(guild)) then
			faction = "kos";
		elseif(UnitExists("mouseover") and UnitIsEnemy("player", "mouseover")) then
			faction = "enemy";
		else
			faction = "friendly";
		end
		
		if(name ~= nil) then
			self:TriggerEvent("VanasKoS_Player_Detected", name, guild, faction, data);
		end

		self:TriggerEvent("VanasKoS_Player_MouseOver", name, guild);
	end
end

function VanasKoSDataGatherer:ChatCombatMessageHandlerFriendly()
	self:ChatCombatMessageHandler("friendly");
end

function VanasKoSDataGatherer:ChatCombatMessageHandlerEnemy()
	self:ChatCombatMessageHandler("enemy");
end

local patternMap = VanasKoS_Parser_MessageMap;
local results = {};

function VanasKoSDataGatherer:GetPlayerOfChatCombatMessage(message)
	if(inBattleground) then
		return;
	end
	local name = nil;
	for index, value in pairs(patternMap) do
		--self:Print("results " .. message .. " " .. value.pattern);
		results[1], results[2], results[3], results[4], results[5], results[6] = message:find(value.pattern);
		if(results[1] ~= nil) then
			local source = value.map['source'];
			if(source and results[source + 2]) then
				name = results[source + 2]:trim();
			else
				local target = value.map['target'];
				if(target ~= nil and results[target + 2]) then
					name = results[target + 2]:trim();
				end
			end
			
			if(name and not name:find(" ")) then
				return name;
			else
				if(VANASKOS.DEBUG == 1) then
					self:Print("[DEBUG]: Parsing: " .. message .. " yielded name with spaces -- name:" .. name .. " -- pattern:" .. value.pattern );
				end
			end
		end
	end
	if(VANASKOS.DEBUG == 1) then
		self:Print("[DEBUG]: Message: " .. message .. " couldn't be parsed");
	end
	return nil;
end

function VanasKoSDataGatherer:ChatMessageAndEmoteHandler()
	if(VanasKoS:IsKoSPlayer(arg2)) then
		self:TriggerEvent("VanasKoS_Player_Detected", arg2, nil, "kos");
	end
end

function VanasKoSDataGatherer:ChatCombatMessageHandler(faction)
  if(arg1) then
  	local name = self:GetPlayerOfChatCombatMessage(arg1);
  	if(name == nil) then
  		return nil;
  	end
	
	name = string.trim(name);
	
    if(VanasKoS:IsKoSPlayer(name)) then
		self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "kos");
    end
    if(faction == "enemy") then
		-- workaround - blizzard flags party members as enemy
		if(GetNumPartyMembers() ~= 0) then
			for i = 1, 4 do
				local partyname = UnitName("party" .. i);
				if(partyname ~= nil and partyname == name) then
					self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "friendly");
					return nil;
				end
			end
		end
		
		self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "enemy");
    end
    if(faction == "friendly") then
		self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "friendly");
    end
  end
end 

