﻿--[[----------------------------------------------------------------------
      Notifier Module - Part of VanasKoS
Notifies the user via Tooltip, Chat and Upper Area of a KoS/other List Target
------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoSNotifier");
VanasKoSNotifier = VanasKoS:NewModule("Notifier");
local VanasKoSNotifier = VanasKoSNotifier;

local FADE_IN_TIME = 0.2;
local FADE_OUT_TIME = 0.2;
local FLASH_TIMES = 1;

L:RegisterTranslations("enUS", function() return {
	["KoS: %s"] = "KoS: %s",
	["KoS (Guild): %s"] = "KoS (Guild): %s",
	["Nicelist: %s"] = "Nicelist: %s",
	["Hatelist: %s"] = "Hatelist: %s",
	["Wanted: %s"] = "Wanted: %s",
	["%sKoS: %s"] = "|cffff00ff%s's|r KoS: %s",
	["%sKoS (Guild): %s"] = "|cffff00ff%s's|r KoS (Guild): %s",
	["%sNicelist: %s"] = "|cffff00ff%s's|r Nicelist: %s",
	["%sHatelist: %s"] = "|cffff00ff%s's|r Hatelist: %s",
	["%sWanted: %s"] = "|cffff00ff%s's|r Wanted: %s",
	
	["Notifications"] = true,
	["Notification in the Upper Area"] = true,
	["Notification in the Chatframe"] = true,
	["Notification through Target Portrait"] = true,
	["Notification through flashing Border"] = true,
	["Notify only on my KoS-Targets"] = true,
	["Notify in Shattrath"] = true,
	["Additional Reason Window"] = true,
	["Locked"] = true,
	
	["Sound on KoS detection"] = true,
	["Notification Interval (seconds)"] = true,
	["Enabled"]  = true,
	["None"] = true,
	["wins: %d - losses: %d"] = "wins: |cff00ff00%d|r losses: |cffff0000%d|r",
	["Show PvP-Stats in Tooltip"] = true,
} end);

L:RegisterTranslations("deDE", function() return {
	["KoS: %s"] = "KoS: %s",
	["KoS (Guild): %s"] = "KoS (Gilde): %s",
	["Nicelist: %s"] = "Nette-Leute Liste: %s",
	["Hatelist: %s"] = "Hassliste: %s",
	["Wanted: %s"] = "Gesucht: %s",
	["%sKoS: %s"] = "|cffff00ff%ss|r KoS: %s",
	["%sKoS (Guild): %s"] = "|cffff00ff%ss|r KoS (Gilde): %s",
	["%sNicelist: %s"] = "|cffff00ff%ss|r Nette-Leute Liste: %s",
	["%sHatelist: %s"] = "|cffff00ff%ss|r Hassliste: %s",
	["%sWanted: %s"] = "|cffff00ff%s's|r Gesucht-Liste: %s",

	["Notifications"] = "Benachrichtigungen",
	["Notification in the Upper Area"] = "Benachrichtigung im oberen Bereich",
	["Notification in the Chatframe"] = "Benachrichtigung im Chat-Fenster",
	["Notification through Target Portrait"] = "Benachrichtigung durch aendern des Ziel-Fensters",
	["Notification through flashing Border"] = "Benachrichtigung durch Aufleuchten des Rahmens",
	["Notify only on my KoS-Targets"] = "Nur bei meinen KoS-Zielen benachrichtigen",
	["Notify in Shattrath"] = "In Shattrath benachrichtigen",
	["Additional Reason Window"] = "Extra Grund Fenster",
	["Locked"] = "Sperren",

	["Notify only on my KoS-Targets"] = "Nur meine KoS-Ziele benachrichtigen",
	["Sound on KoS detection"] = "Audio Benachrichtigung bei KoS",
	["Notification Interval (seconds)"] = "Benachrichtigungs Interval (Sekunden)",
	["Enabled"] = "Aktiviert",
	["None"] = "Keiner",
	["wins: %d - losses: %d"] = "gewonnen: |cff00ff00%d|r verloren: |cffff0000%d|r",
	["Show PvP-Stats in Tooltip"] = "Anzeigen von PvP-Statistiken im Tooltip",
} end);

L:RegisterTranslations("frFR", function() return {
	["KoS: %s"] = "KoS: %s",
	["KoS (Guild): %s"] = "KoS (Guilde): %s",
	["Nicelist: %s"] = "Liste blanche: %s",
	["Hatelist: %s"] = "Liste noire: %s",
--	["Wanted: %s"] = "Wanted: %s",
	["%sKoS: %s"] = "KoS à |cffff00ff%s|r: %s",
	["%sKoS (Guild): %s"] = "KoS (Guilde) à |cffff00ff%s|r: %s",
	["%sNicelist: %s"] = "Liste blanche à |cffff00ff%s|r: %s",
	["%sHatelist: %s"] = "Liste noire à |cffff00ff%s|r: %s",
--	["%sWanted: %s"] = "|cffff00ff%s's|r Wanted: %s",

	["Notifications"] = "Notifications",
	["Notification in the Upper Area"] = "Notification en message principal central",
	["Notification in the Chatframe"] = "Notification dans la fen\195\170tre de discussion",
	["Notification through Target Portrait"] = "Notification avec portrait (dragon élite)",
	["Notification through flashing Border"] = "Notification avec bordure flash",
	["Notify only on my KoS-Targets"] = "Notification de mes propres cibles seulement",
--	["Notify in Shattrath"] = true,
	["Additional Reason Window"] = "Fenêtre additionnelle de raison",
	["Locked"] = "Verrouill\195\169",

	["Sound on KoS detection"] = "Son de d\195\169tection KoS",
	["Notification Interval (seconds)"] = "Intervalle entre notifications (secondes)",
	["Enabled"] = "Actif",
	["None"] = "Aucun",
	["wins: %d - losses: %d"] = "victoires: |cff00ff00%d|r d\195\169faites: |cffff0000%d|r",
	["Show PvP-Stats in Tooltip"] = "Afficher PvP-Stats dans le Tooltip",
} end);

L:RegisterTranslations("koKR", function() return {
	["KoS: %s"] = "KoS: %s",
	["KoS (Guild): %s"] = "KoS (길드): %s",
	["Nicelist: %s"] = "호인명부: %s",
	["Hatelist: %s"] = "악인명부: %s",
	["Wanted: %s"] = "수배: %s",
	["%sKoS: %s"] = "|cffff00ff%s|r의 KoS: %s",
	["%sKoS (Guild): %s"] = "|cffff00ff%s|r의 KoS (길드): %s",
	["%sNicelist: %s"] = "|cffff00ff%s|r의 호인명부: %s",
	["%sHatelist: %s"] = "|cffff00ff%s|r의 악인명부: %s",
	["%sWanted: %s"] = "|cffff00ff%s|r의 수배: %s",

	["Notifications"] = "알림",
	["Notification in the Upper Area"] = "상단 영역에 알림",
	["Notification in the Chatframe"] = "대화창에 알림",
	["Notification through Target Portrait"] = "대상 사진을 통해 알림",
	["Notification through flashing Border"] = "테두리 반짝임을 통해 알림",
	["Notify only on my KoS-Targets"] = "나의 KoS-대상일 경우만 알림",
	["Notify in Shattrath"] = "샤트라스 내 알림",
	["Additional Reason Window"] = "이유창 추가",
	["Locked"] = "고정",

	["Sound on KoS detection"] = "KoS 발견 효과음",
	["Notification Interval (seconds)"] = "알림 간격(초)",
	["Enabled"]  = "사용",
	["None"] = "없음",
	["wins: %d - losses: %d"] = "승: |cff00ff00%d|r 패: |cffff0000%d|r",
	["Show PvP-Stats in Tooltip"] = "툴팁에 PvP-현황 표시",
} end);

local notifyAllowed = true;
local flashNotifyFrame = nil;

function VanasKoSNotifier:UpdateAndCreateReasonFrame()
	if(self.db.profile.notifyExtraReasonFrameEnabled) then
		local reasonFrame = nil;
		if(VanasKoS_Notifier_ReasonFrame ~= nil) then
			reasonFrame = VanasKoS_Notifier_ReasonFrame;
		else
			reasonFrame = CreateFrame("Frame", "VanasKoS_Notifier_ReasonFrame");
		end
		reasonFrame:SetWidth(300);
		reasonFrame:SetHeight(13);
		reasonFrame:SetPoint("CENTER");
		reasonFrame:SetMovable(true);
		reasonFrame:SetToplevel(true);

		local background = nil;
		if(VanasKoS_Notifier_ReasonFrame_Background ~= nil) then
			background = VanasKoS_Notifier_ReasonFrame_Background;
		end

		if(not self.db.profile.notifyExtraReasonFrameLocked) then
			if(background == nil) then
				background = reasonFrame:CreateTexture("VanasKoS_Notifier_ReasonFrame_Background", "BACKGROUND");
			end
			
			background:SetAllPoints();
			background:SetTexture(1.0, 1.0, 1.0, 0.5);
		else
			if(background) then
				background:SetAlpha(0);
			end
		end
		
		local text = nil;
		if(VanasKoS_Notifier_ReasonFrame_Text ~= nil) then
			text = VanasKoS_Notifier_ReasonFrame_Text;
		else
			text = reasonFrame:CreateFontString("VanasKoS_Notifier_ReasonFrame_Text", "OVERLAY");
		end
		
		-- only set the left point, so texts longer than reasonFrame:GetWidth() will show
		text:SetPoint("LEFT", reasonFrame, "LEFT", 0, 0);
		text:SetJustifyH("LEFT");
		text:SetFontObject("GameFontNormalSmall");
		
		if(not self.db.profile.notifyExtraReasonFrameLocked) then
			text:SetTextColor(1.0, 0.0, 1.0);
			text:SetText(self:GetKoSString(nil, "Guild", "MyReason", UnitName("player"), nil, "GuildKoS Reason", UnitName("player"), nil));
		end
		
		reasonFrame:EnableMouse(true);
		reasonFrame:RegisterForDrag("LeftButton");
		reasonFrame:SetScript("OnDragStart", function()
									if(not VanasKoSNotifier.db.profile.notifyExtraReasonFrameLocked) then
										reasonFrame:StartMoving();
									end
							end);
		reasonFrame:SetScript("OnDragStart", function()
									if(not VanasKoSNotifier.db.profile.notifyExtraReasonFrameLocked) then
										reasonFrame:StartMoving();
									end
							end);
		reasonFrame:SetScript("OnDragStop",
								function()
									reasonFrame:StopMovingOrSizing();
								end);
	else
		if(VanasKoS_Notifier_ReasonFrame ~= nil) then
			VanasKoS_Notifier_ReasonFrame:Hide();
		end
	end
end

function VanasKoSNotifier:OnInitialize()
	VanasKoS:RegisterDefaults("Notifier", "profile", {
		Enabled = true,
		notifyVisual = true,
		notifyChatframe = true,
		notifyTargetFrame = true,
		notifyOnlyMyTargets = true,
		notifyFlashingBorder = true,
		notifyInShattrathEnabled = false,
		notifyExtraReasonFrameEnabled = false,
		notifyExtraReasonFrameLocked = false,

		NotifyTimerInterval = 60,
		
		playSound = true,
		playSoundName = "igBonusBarOpen",
		
	});

	self.db = VanasKoS:AcquireDBNamespace("Notifier");

	flashNotifyFrame = CreateFrame("Frame", "VanasKoS_Notifier_Frame", WorldFrame);
	flashNotifyFrame:SetAllPoints();
	flashNotifyFrame:SetToplevel(1);
	flashNotifyFrame:SetAlpha(0);
	
	local texture = flashNotifyFrame:CreateTexture(nil, "BACKGROUND");
	texture:SetTexture("Interface\\AddOns\\VanasKoS\\Artwork\\KoSFrame");
	
	texture:SetBlendMode("ADD");
	texture:SetAllPoints(); -- important! gets set in the blizzard xml stuff implicit, while we have to do it explicit with .lua
	flashNotifyFrame:Hide();
	
	self:UpdateAndCreateReasonFrame();
	
	VanasKoSGUI:AddConfigOption("Notifier", {
		type = 'group',
		name = L["Notifications"],
		desc = L["Notifications"],
		args = {
			enabled = {
				type = 'toggle',
				name = L["Enabled"],
				desc = L["Enabled"],
				order = 1,
				set = function(v) VanasKoSNotifier.db.profile.Enabled = v; VanasKoS:ToggleModuleActive("Notifier", v); end,
				get = function() return VanasKoSNotifier.db.profile.Enabled; end,
			},
			upperarea = {
				type = 'toggle',
				name = L["Notification in the Upper Area"],
				desc = L["Notification in the Upper Area"],
				order = 2,
				set = function(v) VanasKoSNotifier.db.profile.notifyVisual = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyVisual; end,
			},
			chatframe = {
				type = 'toggle',
				name = L["Notification in the Chatframe"],
				desc = L["Notification in the Chatframe"],
				order = 3,
				set = function(v) VanasKoSNotifier.db.profile.notifyChatframe = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyChatframe; end,
			},
			targetframe = {
				type = 'toggle',
				name = L["Notification through Target Portrait"],
				desc = L["Notification through Target Portrait"],
				order = 4,
				set = function(v) VanasKoSNotifier.db.profile.notifyTargetFrame = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyTargetFrame; end,
			},
			flashingborder = {
				type = 'toggle',
				name = L["Notification through flashing Border"],
				desc = L["Notification through flashing Border"],
				order = 5,
				set = function(v) VanasKoSNotifier.db.profile.notifyFlashingBorder = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyFlashingBorder; end,
			},
			onlymytargets = {
				type = 'toggle',
				name = L["Notify only on my KoS-Targets"],
				desc = L["Notify only on my KoS-Targets"],
				order = 6,
				set = function(v) VanasKoSNotifier.db.profile.notifyOnlyMyTargets = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyOnlyMyTargets; end,
			},
			inshattrath = {
				type = 'toggle',
				name = L["Notify in Shattrath"],
				desc = L["Notify in Shattrath"],
				order = 7,
				set = function(v) VanasKoSNotifier.db.profile.notifyInShattrathEnabled = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyInShattrathEnabled; end,
			},
			showpvpstats = {
				type = 'toggle',
				name = L["Show PvP-Stats in Tooltip"],
				desc = L["Show PvP-Stats in Tooltip"],
				order = 8,
				set = function(v) VanasKoSNotifier.db.profile.notifyShowPvPStats = v; end,
				get = function() return VanasKoSNotifier.db.profile.notifyShowPvPStats; end,
			},
			showextrareasonframe = {
				type = 'group',
				name = L["Additional Reason Window"],
				desc = L["Additional Reason Window"],
				order = 9,
				args = {
					enabled = {
						type = 'toggle',
						name = L["Enabled"],
						desc = L["Enabled"],
						order = 1,
						set = function(v) VanasKoSNotifier.db.profile.notifyExtraReasonFrameEnabled = v; VanasKoSNotifier:UpdateAndCreateReasonFrame(); end,
						get = function() return VanasKoSNotifier.db.profile.notifyExtraReasonFrameEnabled; end,
					},
					locked = {
						type = 'toggle',
						name = L["Locked"],
						desc = L["Locked"],
						order = 1,
						set = function(v) VanasKoSNotifier.db.profile.notifyExtraReasonFrameLocked = v; VanasKoSNotifier:UpdateAndCreateReasonFrame(); end,
						get = function() return VanasKoSNotifier.db.profile.notifyExtraReasonFrameLocked; end,
					}
				}
			},
			notificationInterval = {
				type = 'range',
				name = L["Notification Interval (seconds)"],
				desc = L["Notification Interval (seconds)"],
				min = 0,
				max = 600,
				step = 5,
				order = 10,
				set = function(value) VanasKoSNotifier.db.profile.NotifyTimerInterval = value; end,
				get = function() return VanasKoSNotifier.db.profile.NotifyTimerInterval; end,
			},
			sound = {
				type = 'group',
				name = L["Sound on KoS detection"],
				desc = L["Sound on KoS detection"],
				order = 11,
				args = {
					none = {
						type = 'toggle',
						name = L["None"],
						desc = L["None"],
						order = 1,
						set = function(value) VanasKoSNotifier.db.profile.playSound = false; end,
						get = function() return not VanasKoSNotifier.db.profile.playSound; end,
						isRadio = true,
					},
					sound1 = {
						type = 'toggle',
						name = "Bonus Bar",
						desc = "Bonus Bar",
						set = function(value) VanasKoSNotifier.db.profile.playSound = true; VanasKoSNotifier.db.profile.playSoundName = "igBonusBarOpen"; VanasKoSNotifier:PlaySound(); end,
						get = function() 
									if(VanasKoSNotifier.db.profile.playSound and VanasKoSNotifier.db.profile.playSoundName == "igBonusBarOpen") then 
										return true;
									else 
										return false; 
									end 
								end,
						isRadio = true
					},
					sound2 = {
						type = 'toggle',
						name = "Auction House",
						desc = "Auction House",
						set = function(value) VanasKoSNotifier.db.profile.playSound = true; VanasKoSNotifier.db.profile.playSoundName = "AuctionWindowOpen"; VanasKoSNotifier:PlaySound(); end,
						get = function() 
									if(VanasKoSNotifier.db.profile.playSound and VanasKoSNotifier.db.profile.playSoundName == "AuctionWindowOpen") then 
										return true;
									else 
										return false; 
									end 
								end,
						isRadio = true
					},
					sound3 = {
						type = 'toggle',
						name = "String Fading",
						desc = "String Fading",
						set = function(value) VanasKoSNotifier.db.profile.playSound = true; VanasKoSNotifier.db.profile.playSoundName = "Interface\\AddOns\\VanasKoS\\Artwork\\StringFading.mp3"; VanasKoSNotifier:PlaySound(); end,
						get = function() 
									if(VanasKoSNotifier.db.profile.playSound and VanasKoSNotifier.db.profile.playSoundName == "Interface\\AddOns\\VanasKoS\\Artwork\\StringFading.mp3") then 
										return true;
									else 
										return false; 
									end 
								end,
						isRadio = true,
					}
				}
			},
		}
	});
end

function VanasKoSNotifier:OnEnable()
	if(not VanasKoSNotifier.db.profile.Enabled) then
		return;
	end
	self:RegisterEvent("VanasKoS_Player_Detected", "Player_Detected");
	self:RegisterEvent("VanasKoS_Player_Target_Changed", "Player_Target_Changed");
	self:RegisterEvent("VanasKoS_Mob_Target_Changed", "Player_Target_Changed");
	self:RegisterEvent("VanasKoS_Player_MouseOver", "Player_MouseOver");
end

function VanasKoSNotifier:OnDisable()
	self:UnregisterAllEvents();
end

function VanasKoSNotifier:Tooltip_AddMessage(msg)
	GameTooltip:AddLine(msg);
	GameTooltip:Show();
--[[
	-- resize
	local newWidth = 0;
	local numlines = GameTooltip:NumLines();
	GameTooltip:SetHeight((numlines*15) + 15);

	for i = 1, numlines do
        local checkText = getglobal("GameTooltipTextLeft"..i);
        if (checkText and ((checkText:GetWidth() + 24) > newWidth)) then
                newWidth = (checkText:GetWidth() + 24);
        end
	end
	GameTooltip:SetWidth(newWidth); ]]--
end

local listsToCheck = {
		['PLAYERKOS'] = { L["KoS: %s"], L["%sKoS: %s"] },
		['GUILDKOS'] = { L["KoS (Guild): %s"], L["%sKoS (Guild): %s"] },
		['NICELIST'] = { L["Nicelist: %s"], L["%sNicelist: %s"] },
		['HATELIST'] = { L["Hatelist: %s"], L["%sHatelist: %s"] },
		['WANTED'] = {  L["Wanted: %s"], L["%sWanted: %s"] },
	};

function VanasKoSNotifier:Player_MouseOver(name, guild, faction)
	for k,v in pairs(listsToCheck) do
		local reason, creator, owner;
		if(k ~= "GUILDKOS") then
			reason, creator, owner = VanasKoS:IsOnList(k, name);
		else
			reason, creator, owner = VanasKoS:IsOnList(k, guild);
		end
		if(reason ~= nil) then
			if(owner == nil) then
				self:Tooltip_AddMessage(format(v[1], reason));
			else
				self:Tooltip_AddMessage(format(v[2], creator, reason));
			end
		end
	end
	if(self.db.profile.notifyShowPvPStats) then
		local wins, losses = VanasKoS:IsOnList("PVPSTATS", name);
		if(wins == nil) then
			wins = 0;
		end
		if(losses == nil) then
			losses = 0;
		end
		if(wins ~= 0 or losses ~= 0) then
			self:Tooltip_AddMessage(format(L["wins: %d - losses: %d"], wins, losses));
		end
	end
end

function VanasKoSNotifier:UpdateReasonFrame(name, guild)
	if(self.db.profile.notifyExtraReasonFrameEnabled) then
		if(UnitIsPlayer("target")) then
			if(not VanasKoS_Notifier_ReasonFrame_Text) then
				return;
			end
			local reason, creator, owner = VanasKoS:IsOnList("PLAYERKOS", name);
			local greason, gcreator, gowner = VanasKoS:IsOnList("GUILDKOS", guild);
			
			if(reason ~= nil or greason ~= nil) then
				VanasKoS_Notifier_ReasonFrame_Text:SetTextColor(1.0, 0.81, 0.0, 1.0);
				VanasKoS_Notifier_ReasonFrame_Text:SetText(self:GetKoSString(name, guild, reason, creator, owner, greason, gcreator, gowner));
				return;
			end

			local hreason, hcreator, howner = VanasKoS:IsOnList("HATELIST", name);
			if(hreason ~= nil) then
				VanasKoS_Notifier_ReasonFrame_Text:SetTextColor(1.0, 0.0, 0.0, 1.0);
				if(hcreator ~= nil and howner ~= nil)  then
					VanasKoS_Notifier_ReasonFrame_Text:SetText(format(L["%sHatelist: %s"], hcreator, hreason));
				else
					VanasKoS_Notifier_ReasonFrame_Text:SetText(format(L["Hatelist: %s"], hreason));
				end
				return;
			end

			local nreason, ncreator, nowner = VanasKoS:IsOnList("NICELIST", name);
			if(nreason ~= nil) then
				VanasKoS_Notifier_ReasonFrame_Text:SetTextColor(0.0, 1.0, 0.0, 1.0);
				if(ncreator ~= nil and nowner ~= nil)  then
					VanasKoS_Notifier_ReasonFrame_Text:SetText(format(L["%sNicelist: %s"], ncreator, nreason));
				else
					VanasKoS_Notifier_ReasonFrame_Text:SetText(format(L["Nicelist: %s"], nreason));
				end
				return;
			end

			VanasKoS_Notifier_ReasonFrame_Text:SetText("");
		else
			VanasKoS_Notifier_ReasonFrame_Text:SetText("");
		end
		
	end
end

function VanasKoSNotifier:Player_Target_Changed(name, guild, faction, data)
	if(self.db.profile.notifyTargetFrame) then
		if(UnitIsPlayer("target")) then
			if(VanasKoS:IsKoSPlayer(name)) then
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");
				TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());
			elseif(VanasKoS:IsKoSGuild(guild)) then
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare");
				TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());
			elseif(VanasKoS:IsOnList("HATELIST", name)) then
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare");
				TargetFrameTexture:SetVertexColor(1.0, 0.0, 0.0, TargetFrameTexture:GetAlpha());
			elseif(VanasKoS:IsOnList("NICELIST", name)) then
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare");
				TargetFrameTexture:SetVertexColor(0.0, 1.0, 0.0, TargetFrameTexture:GetAlpha());
			else
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame");
				TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());
			end
		else
			TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());
		end
	end
	
	self:UpdateReasonFrame(name, guild, faction, data);
end
--/script VanasKoS:TriggerEvent("VanasKoS_Player_Detected", "Apfelherz", nil, "kos");
function VanasKoSNotifier:GetKoSString(name, guild, reason, creator, owner, greason, gcreator, gowner)
	local msg = "";

	if(reason ~= nil) then
		if(creator ~= nil and owner ~= nil) then
			if(name == nil) then
				msg = format(L["%sKoS: %s"], creator, reason);
			else
				msg = format(L["%sKoS: %s"], creator, name .. " (" .. reason .. ")");
			end
		else
			if(name == nil) then
				msg = format(L["KoS: %s"], reason);
			else
				msg = format(L["KoS: %s"], name .. " (" .. reason .. ")");
			end
		end
		if(guild) then
			msg = msg .. " <" .. guild .. ">";
			if(greason ~= nil) then
				msg = msg .. " (" .. greason .. ")";
			end
		end
	elseif(greason ~= nil) then
		msg = format(L["KoS (Guild): %s"], name .. " <" .. guild .. "> (" ..  greason .. ")");
	end
	
	return msg;
end

function VanasKoSNotifier:Player_Detected(name, guild, faction, data)
	if (notifyAllowed ~= true) then
		return nil;
	end
	
	local reason, creator, owner = VanasKoS:IsOnList("PLAYERKOS", name);
	local greason, gcreator, gowner = VanasKoS:IsOnList("GUILDKOS", guild);
	
	if(reason == nil and greason == nil) then
		return;
	end

	-- don't notify if we're in shattrah
	if((reason ~= nil or greason ~= nil) and VanasKoSDataGatherer:IsInShattrath()) then
		if(not self.db.profile.notifyInShattrathEnabled) then
			return;
		end
	end
		
	local msg = self:GetKoSString(name, guild, reason, creator, owner, greason, gcreator, gowner)
	
	if(self.db.profile.notifyOnlyMyTargets) then
		if(owner ~= nil or gowner ~= nil) then
			return;
		end
	end
	
	notifyAllowed = false;
	-- Reallow Notifies in NotifyTimeInterval Time
	self:ScheduleEvent(function() notifyAllowed = true end, self.db.profile.NotifyTimerInterval);
	
	if(self.db.profile.notifyVisual) then
		UIErrorsFrame:AddMessage(msg, 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
	end
	if(self.db.profile.notifyChatframe) then
		VanasKoS:Print(msg);
	end
	if(self.db.profile.notifyFlashingBorder) then
		self:FlashNotify();
	end
	if(self.db.profile.playSound) then
		self:PlaySound();
	end
end

-- /script VanasKoSNotifier:FlashNotify()
function VanasKoSNotifier:FlashNotify()
	flashNotifyFrame:Show();
	UIFrameFlash(VanasKoS_Notifier_Frame, FADE_IN_TIME, FADE_OUT_TIME, FLASH_TIMES*(FADE_IN_TIME + FADE_OUT_TIME));
end

function VanasKoSNotifier:PlaySound()
	local soundname = self.db.profile.playSoundName;
	if(soundname == nil or soundname == "") then
		soundname = "igBonusBarOpen";
	end

	if(string.find(soundname, "\\")) then
		PlaySoundFile(soundname);
	else
		PlaySound(soundname);
	end
end
